precision highp float;

uniform vec2 resolution;

uniform vec4 effectColor;
uniform vec2 effectWeight;

uniform sampler2D u_texture;
varying vec2 v_texCoords;


const float PI_2 = 6.28318;
const float PASS_COUNT = 16.0;
const float QUALITY_STEPS = 4.0;
const float BLUR_FALLOFF_COEFF = 0.4;


void main() {
    vec2 uv = v_texCoords.xy;
    vec2 weight = effectWeight.xy/resolution.xy;

    vec4 outColor = texture2D(u_texture, uv);

    // Blur calculations
    for (float i = 0.0; i < PI_2; i += PI_2 / PASS_COUNT) {
        for(float s = 1.0 / QUALITY_STEPS; s <= 1.0; s += 1.0 / QUALITY_STEPS) {
            outColor += texture2D(u_texture, uv + vec2(cos(i), sin(i)) * weight * s);
        }
    }

    // Output to screen
    outColor.rgb /= PASS_COUNT * QUALITY_STEPS * BLUR_FALLOFF_COEFF;
    outColor.a /= PASS_COUNT * QUALITY_STEPS;
    gl_FragColor = outColor;
}