precision lowp float;

/* smoothtext.frag
 * A shader for drawing text and other partially-transparent textures while avoiding blending artifacts.
 * This is primarily intended for text, because it will override the RGB channels of the texture with
 * v_color. Note that this is not a true pre-multipled alpha shader, but it does address the same issues
 * and matches the pre-Facer 7.0.9 behavior.
 *
 * @author Matthew DiBernardo [07.27.2023]
 */

varying vec2 v_texCoords;

varying vec4 v_color;

uniform sampler2D u_texture;

const float BLEND_FACTOR = 1.0/2.5;

float blend(float val, float factor) {
    return smoothstep(0.5 - factor, 0.5 + factor, val);
}

void main() {
    vec4 texColor = texture2D(u_texture, v_texCoords);
    gl_FragColor = vec4(v_color.rgb, blend(texColor.a, BLEND_FACTOR) * v_color.a);
}