precision highp float;

uniform vec2 resolution;

uniform vec4 effectColor;
uniform vec2 effectWeight;

uniform sampler2D u_texture;
varying vec2 v_texCoords;


const vec4 BKG = vec4(0.0, 0.0, 0.0, 0.0);

const float PI_2 = 6.28318;
const float PASS_COUNT = 16.0;

const float WEIGHT_MULT = 0.6;

void main() {
    vec2 uv = v_texCoords.xy;
    vec2 weight = effectWeight.xy / resolution.xy;

    vec4 outColor = texture2D(u_texture, uv);
    vec4 fullAlphaEffectColor = effectColor;
    fullAlphaEffectColor.a = 1.0;


    // Iterate over segments of a circle
    for (float i = 0.0; i < PI_2; i += PI_2 / PASS_COUNT) {
        // Sample the texture based on the offset
        vec2 offset = vec2(sin(i), cos(i)) * weight * WEIGHT_MULT;
        vec4 color = texture2D(u_texture, uv + offset);

        // Draw the outline based on the alpha of the texture at the current position.
        float alpha = smoothstep(0.1, 0.9, distance(color.a, BKG.a));
        outColor = mix(outColor, fullAlphaEffectColor, alpha);
    }
    ///* Debugging: Draw the original texture
    //vec4 mat = texture2D(u_texture, uv);
    //float factor = smoothstep(0.5, 0.7, distance(mat.a, BKG.a));
    //outColor = mix(outColor, mat, factor);
    //*/

    outColor.a *= effectColor.a;


    // Output the final color
    gl_FragColor = outColor;
}